/** The name of the window pane component. */
echopoint.constants.SP_WINDOW_PANE = "echopoint.consultas.SpWindowPane";

/**
 * The client-side representation of the Echo2Consultas window pane
 * component.  The component supports minimising the window pane to the
 * content pane similar to MDI.
 *
 * @author Rakesh 2010-04-21
 * @version $Id: Application.FileUploadSelector.js 183 2009-05-04 14:48:36Z sptrakesh $
 */
echopoint.consultas.SpWindowPane = Core.extend( Echo.Component,
{
  $load: function()
  {
    Echo.ComponentFactory.registerType( echopoint.constants.SP_WINDOW_PANE, this );
  },

  $static:
  {
    /**
     * Default WindowPane border.
     * @type #FillImageBorder
     */
    DEFAULT_BORDER: { color: "#36537a", borderInsets: 20, contentInsets: 3 },

    /**
     * Default WindowPane content background color.
     * @type #Color
     */
    DEFAULT_BACKGROUND: "#ffffff",

    /**
     * Default WindowPane content background color.
     * @type #Color
     */
    DEFAULT_FOREGROUND: "#000000",

    /**
     * Default insets around WindowPane controls.
     * @type #Insets
     */
    DEFAULT_CONTROLS_INSETS: 4,

    /**
     * Default spacing between WindowPane controls.
     * @type #Extent
     */
    DEFAULT_CONTROLS_SPACING: 4,

    /**
     * Default WindowPane height.
     * @type #Extent
     */
    DEFAULT_HEIGHT: "15em",

    /**
     * Default WindowPane minimum width.
     * @type #Extent
     */
    DEFAULT_MINIMUM_WIDTH: 100,

    /**
     * Default WindowPane minimum height.
     * @type #Extent
     */
    DEFAULT_MINIMUM_HEIGHT: 100,

    /**
     * Default WindowPane title background color.
     * @type #Color
     */
    DEFAULT_TITLE_BACKGROUND: "#becafe",

    /**
     * Default WindowPane title height.
     * @type #Extent
     */
    DEFAULT_TITLE_HEIGHT: 30,

    /**
     * Default WindowPane title insets.
     * @type #Insets
     */
    DEFAULT_TITLE_INSETS: "5px 10px",

    /**
     * Default WindowPane width.
     * @type #Extent
     */
    DEFAULT_WIDTH: "30em"
  },

  componentType: echopoint.constants.SP_WINDOW_PANE,

  /** @see Echo.Component#modalSupport */
  modalSupport: true,

  /**
   * Render as floating pane in ContentPanes.
   * @see Echo.ContentPane
   */
  floatingPane: true,

  /** @see Echo.Component#pane */
  pane: true,

  /** @see Echo.Component#focusable */
  focusable: true,

  $virtual:
  {
    /**
     * Object specifying state of window pane before it was maximized,
     * May contain x, y, width, height integer properties or be null.
     */
    originalState: null,

    /** A flag indicating that the window pane is minimised. */
    minimized: false,

    /** The dock manager for managing minimised instances. */
    dockManager: null
  },

  /** Processes a user request to close the window. */
  userClose: function()
  {
    this.fireEvent( {type: "close", source: this} );
  },

  /** Processes a user request to maximize the window. */
  userMaximize: function()
  {
    if ( this.render( "width" ) == "100%" && this.render( "height" ) == "100%" )
    {
      if ( this.originalState )
      {
        this.set( "width", this.originalState.width );
        this.set( "height", this.originalState.height );
        this.set( "positionX", this.originalState.x );
        this.set( "positionY", this.originalState.y );
      }
    }
    else
    {
      if ( this.minimized )
      {
        this.peer.maximize();
      }
      else
      {
        this.originalState = {
          x: this.get( "positionX" ), y: this.get( "positionY" ),
          width: this.get( "width" ), height: this.get( "height" ) };
        this.set( "width", "100%" );
        this.set( "height", "100%" );
      }
    }

    this.fireEvent( {type: "maximize", source: this} );
  }
} );
